/**
 * @file
 * Protected Content encryption of text.
 */
import {setCloseDialogBtnProcFieldClass} from './proc-field-nested-dialog-module.js';
(async function($, Drupal, once) {
  Drupal.behaviors.ProcBehavior = {
    async attach(context, settings) {
      if (context.id !== '') {
        async function encrypt() {
          const procJsLabels = drupalSettings.proc_labels;

          if (document.getElementsByName('proc-text-field').length) {
            const plaintext = document.getElementsByName('proc-text-field')[0]
              .value;

            if (plaintext.length === 0) {
              return;
            }

            const plainTextSize = new TextEncoder().encode(plaintext).length;
            const postMaxSizeBytes = parseInt(
              drupalSettings.proc_post_max_size_bytes,
              10,
            );
            const fileEntityMaxSize = parseInt(
              drupalSettings.proc_file_entity_max_filesize,
              10,
            );
            const dynamicMaximumSize = postMaxSizeBytes / 4;
            let realMaxSize = dynamicMaximumSize;

            if (
              plainTextSize > dynamicMaximumSize ||
              plainTextSize > fileEntityMaxSize
            ) {
              if (fileEntityMaxSize < dynamicMaximumSize) {
                realMaxSize = fileEntityMaxSize;
              }

              messages.add(
                `${procJsLabels.proc_max_encryption_size} ${realMaxSize} ${procJsLabels.proc_max_encryption_size_unit}`,
                {
                  type: 'error',
                },
              );
              return;
            }

            const recipientsPubkeys = [];
            const recipientsUidsKeysChanged = JSON.parse(
              drupalSettings.proc_recipients_pubkeys_changed,
            );
            const remoteKey = [];

            openpgp.config.showComment = false;
            openpgp.config.showVersion = false;

            for (const userIdIterator in recipientsUidsKeysChanged) {
              const localKey = localStorage.getItem(
                `proc.key_user_id.${userIdIterator}.${recipientsUidsKeysChanged[userIdIterator]}`,
              );

              if (localKey) {
                console.info('Key found in local storage');
                recipientsPubkeys.push(localKey);
              } else {
                const storageKeys = Object.keys(localStorage);

                if (storageKeys.length > 0) {
                  storageKeys.forEach(function(storageKey, storageKeyIndex) {
                    if (
                      storageKey.startsWith(
                        `proc.key_user_id.${userIdIterator}`,
                      )
                    ) {
                      localStorage.removeItem(storageKeys[storageKeyIndex]);
                    }
                  });
                }

                remoteKey.push(userIdIterator);
              }
            }

            let pubkeysJson;

            if (remoteKey.length > 0) {
              const remoteKeyCsv = remoteKey.join(',');

              const pubKeyAjax = async remoteKeyCsv => {
                const response = await fetch(
                  `${window.location.origin +
                    drupalSettings.basePath}api/proc/getpubkey/${remoteKeyCsv}/user_id`,
                );
                pubkeysJson = await response.json();

                if (pubkeysJson.pubkey.length > 0) {
                  pubkeysJson.pubkey.forEach(function(pubkey, index) {
                    recipientsPubkeys.push(pubkey.key);

                    try {
                      localStorage.setItem(
                        `proc.key_user_id.${remoteKey[index]}.${pubkey.changed}`,
                        pubkey.key,
                      );
                    } catch (error) {
                      console.warn(error);
                    }
                  });
                }
              };

              await pubKeyAjax(remoteKeyCsv);
            }

            const publicKeys = await Promise.all(
              recipientsPubkeys.map(armoredKey =>
                openpgp.readKey({
                  armoredKey,
                }),
              ),
            );

            const plainText = document.getElementsByName('proc-text-field')[0]
              .value;
            const startSeconds = new Date().getTime() / 1000;
            const message = await openpgp.createMessage({
              text: plainText,
            });

            const encrypted = await openpgp.encrypt({
              encryptionKeys: publicKeys,
              message,
              format: 'armored',
              config: {
                preferredCompressionAlgorithm: openpgp.enums.compression.zip,
              },
            });

            const endSeconds = new Date().getTime() / 1000;
            const total = endSeconds - startSeconds;

            if (encrypted) {
              setCloseDialogBtnProcFieldClass();
              $('input[name=cipher_text]')[0].value = encrypted;
              $('input[name=source_file_name]')[0].value = '*******';
              $('input[name=source_file_size]')[0].value = plainTextSize;
              $('input[name=source_file_type]')[0].value = 'text/plain';
              $('input[name=source_file_last_change]')[0].value = '0';
              $(
                'input[name=browser_fingerprint]',
              )[0].value = `${navigator.userAgent}, (${screen.width} x ${screen.height})`;
              $('input[name=generation_timestamp]')[0].value = startSeconds;
              $('input[name=generation_timespan]')[0].value = total;
              $('input[name=signed]')[0].value = 0;

              if ($('input[name=cipher_text]')[0].value) {
                document.getElementsByName('proc-text-field')[0].value = '';

                document.querySelectorAll('[id^=edit-submit-proc]')[0].click();
                const field = $(
                  `#${drupalSettings.proc_form_triggering_field}`,
                );

                const fieldLocator = $(
                  `#${drupalSettings.proc_form_triggering_field}`,
                );
                // If there is already a clone, remove it:
                if (
                  document.getElementById(
                    `${drupalSettings.proc_form_triggering_field}--clone`,
                  )
                ) {
                  document
                    .getElementById(
                      `${drupalSettings.proc_form_triggering_field}--clone`,
                    )
                    .remove();
                }

                field[0].setAttribute('data-proc-plaintext', plainText);

                const fieldClone = field
                  .clone()
                  .attr(
                    'id',
                    `${drupalSettings.proc_form_triggering_field}--clone`,
                  );

                fieldClone.attr(
                  'name',
                  `${drupalSettings.proc_form_triggering_field}--name-clone`,
                );

                fieldClone.attr('style', 'display:block;');
                fieldClone.attr('data-plaintext', '');
                fieldClone.attr('disabled', 'disabled');
                fieldClone.insertAfter(fieldLocator);
                field[0].style.display = 'none';
                document.getElementById(
                  `${drupalSettings.proc_form_triggering_field}--clone`,
                ).value = '*******';
              }
            }
          }
        }

        // Get the plain text from the main form:
        let formPlaintext = 0;

        if (drupalSettings.proc_form_triggering_field) {
          if (
            document.getElementById(drupalSettings.proc_form_triggering_field)
              .value
          ) {
            formPlaintext = document.getElementById(
              drupalSettings.proc_form_triggering_field,
            ).value;

            if (formPlaintext != 0) {
              if (document.getElementsByName('proc-text-field')[0]) {
                document.getElementsByName(
                  'proc-text-field',
                )[0].value = formPlaintext;
                encrypt();
                return;
              }
            }
          }
        }

        if (document.getElementById('edit-submit-proc')) {
          document
            .getElementById('edit-submit-proc')
            .addEventListener('click', encrypt, false);
        }
      }
    },
  };
})(jQuery, Drupal, once);
